@@ -648,7 +648,7 @@ StructureBase* House::placeStructure(Uint32 builderID, int itemID, int xPos, int
 
     if(currentGame->getGameInitSettings().getGameOptions().onlyOnePalace && pBuilder != nullptr && itemID == Structure_Palace && getNumItems(Structure_Palace) > 0) {
         if(this == pLocalHouse && pBuilder->isSelected()) {
-            currentGame->currentCursorMode = Game::CursorMode_Normal;
+            currentGame->setCursorMode(Game::CursorMode_Normal);
         }
         return nullptr;
     }
@@ -666,7 +666,7 @@ StructureBase* House::placeStructure(Uint32 builderID, int itemID, int xPos, int
 
                 if(this == pLocalHouse) {
                     if(pBuilder->isSelected()) {
-                        currentGame->currentCursorMode = Game::CursorMode_Normal;
+                        currentGame->setCursorMode(Game::CursorMode_Normal);
                     }
 
                     pLocalPlayer->onPlaceStructure(nullptr);
@@ -695,7 +695,7 @@ StructureBase* House::placeStructure(Uint32 builderID, int itemID, int xPos, int
 
                 if(this == pLocalHouse) {
                     if(pBuilder->isSelected()) {
-                        currentGame->currentCursorMode = Game::CursorMode_Normal;
+                        currentGame->setCursorMode(Game::CursorMode_Normal);
                     }
 
                     pLocalPlayer->onPlaceStructure(nullptr);
@@ -763,7 +763,7 @@ StructureBase* House::placeStructure(Uint32 builderID, int itemID, int xPos, int
 
                 if (this == pLocalHouse) {
                     if(pBuilder->isSelected()) {
-                        currentGame->currentCursorMode = Game::CursorMode_Normal;
+                        currentGame->setCursorMode(Game::CursorMode_Normal);
                     }
                     pLocalPlayer->onPlaceStructure(newStructure);
                 }
@@ -915,16 +915,3 @@ void House::decrementHarvesters() {
         }
     }
 }
-
-int House::calculateMilitaryValue() const {
-    int value = 0;
-    for (Uint32 i = Unit_FirstID; i <= Unit_LastID; i++) {
-        if (i != Unit_Carryall
-            && i != Unit_Harvester
-            && i != Unit_MCV
-            && i != Unit_Sandworm) {
-            value += getNumItems(i) * currentGame->objectData.data[i][houseID].price;
-        }
-    }
-    return value;
-}
