@@ -222,25 +222,17 @@ Coord ObjectBase::getClosestCenterPoint(const Coord& objectLocation) const {
 
 
 int ObjectBase::getMaxHealth() const {
-    return currentGame->objectData.data[itemID][originalHouseID].hitpoints;
+    if(currentGame != nullptr) {
+        return currentGame->objectData.data[itemID][originalHouseID].hitpoints;
+    }
+    return 100; // Default fallback health
 }
 
 void ObjectBase::handleDamage(int damage, Uint32 damagerID, House* damagerOwner) {
     if(damage >= 0) {
         FixPoint newHealth = getHealth();
-        newHealth -= damage;
 
-        // Track damage dealt by the attacker
-        if(damagerOwner != nullptr) {
-            ObjectBase* pDamager = currentGame->getObjectManager().getObject(damagerID);
-            if(pDamager != nullptr) {
-                // If you damaged your own unit/structure then damage should be negative
-                if(damagerOwner == getOwner()) {
-                    damage *= -1;
-                }
-                damagerOwner->informHasDamaged(pDamager->getItemID(), damage);
-            }
-        }
+        newHealth -= damage;
 
         if(newHealth <= 0) {
             setHealth(0);
@@ -268,9 +260,11 @@ ObjectInterface* ObjectBase::getInterfaceContainer() {
 }
 
 void ObjectBase::removeFromSelectionLists() {
-    currentGame->getSelectedList().erase(getObjectID());
-    currentGame->selectionChanged();
-    currentGame->getSelectedByOtherPlayerList().erase(getObjectID());
+    if(currentGame != nullptr) {
+        currentGame->getSelectedList().erase(getObjectID());
+        currentGame->selectionChanged();
+        currentGame->getSelectedByOtherPlayerList().erase(getObjectID());
+    }
     selected = false;
 }
 
@@ -451,15 +445,15 @@ const ObjectBase* ObjectBase::findClosestTarget() const {
 }
 
 const ObjectBase* ObjectBase::findTarget() const {
-//searches for a target in an area like as shown below
-//
-//                    *
-//                  *****
-//                  *****
-//                 ***T***
-//                  *****
-//                  *****
-//                    *
+    //searches for a target in an area like as shown below
+    //
+    //                    *
+    //                  *****
+    //                  *****
+    //                 ***T***
+    //                  *****
+    //                  *****
+    //                    *
 
     auto checkRange = 0;
     switch(attackMode) {
@@ -493,38 +487,62 @@ const ObjectBase* ObjectBase::findTarget() const {
     ObjectBase *pClosestTarget = nullptr;
     auto closestTargetDistance = FixPt_MAX;
 
-    Coord coord;
-    const auto startY = std::max(0, location.y - checkRange);
-    const auto endY = std::min(currentGameMap->getSizeY()-1, location.y + checkRange);
-    for(coord.y = startY; coord.y <= endY; coord.y++) {
-        const auto startX = std::max(0, location.x - checkRange);
-        const auto endX = std::min(currentGameMap->getSizeX()-1, location.x + checkRange);
-        for(coord.x = startX; coord.x <= endX; coord.x++) {
+    // Start from center and expand outward in rings
+    for(auto ring = 0; ring <= checkRange; ring++) {
+        // Check each point in the current ring
+        for(auto x = -ring; x <= ring; x++) {
+            for(auto y = -ring; y <= ring; y++) {
+                // Only check points on the ring's perimeter
+                if(std::abs(x) != ring && std::abs(y) != ring) {
+                    continue;
+                }
 
-            const auto targetDistance = blockDistance(location, coord);
-            if(targetDistance <= checkRange) {
-                Tile* pTile = currentGameMap->getTile(coord);
-                if( pTile->isExploredByTeam(getOwner()->getTeamID())
-                    && !pTile->isFoggedByTeam(getOwner()->getTeamID())
-                    && pTile->hasAnObject()) {
+                const auto checkX = location.x + x;
+                const auto checkY = location.y + y;
 
-                    const auto pNewTarget = pTile->getObject();
-                    if(((pNewTarget->getItemID() != Structure_Wall && pNewTarget->getItemID() != Unit_Carryall) || pClosestTarget == nullptr) && canAttack(pNewTarget)) {
-                        if(targetDistance < closestTargetDistance) {
-                            pClosestTarget = pNewTarget;
-                            closestTargetDistance = targetDistance;
+                // Skip if out of bounds
+                if(checkX < 0 || checkX >= currentGameMap->getSizeX() ||
+                   checkY < 0 || checkY >= currentGameMap->getSizeY()) {
+                    continue;
+                }
+
+                const auto targetDistance = blockDistance(location, Coord(checkX, checkY));
+                if(targetDistance <= checkRange) {
+                    Tile* pTile = currentGameMap->getTile(Coord(checkX, checkY));
+                    if(pTile->isExploredByTeam(getOwner()->getTeamID()) &&
+                       !pTile->isFoggedByTeam(getOwner()->getTeamID()) &&
+                       pTile->hasAnObject()) {
+
+                        const auto pNewTarget = pTile->getObject();
+                        if(((pNewTarget->getItemID() != Structure_Wall && 
+                             pNewTarget->getItemID() != Unit_Carryall) || 
+                            pClosestTarget == nullptr) && 
+                           canAttack(pNewTarget)) {
+                            if(targetDistance < closestTargetDistance) {
+                                pClosestTarget = pNewTarget;
+                                closestTargetDistance = targetDistance;
+                            }
                         }
                     }
                 }
             }
         }
+
+        // If we found a target in this ring, we can stop searching
+        // as we won't find a closer one in outer rings
+        if(pClosestTarget != nullptr) {
+            break;
+        }
     }
 
     return pClosestTarget;
 }
 
 int ObjectBase::getViewRange() const {
-    return currentGame->objectData.data[itemID][originalHouseID].viewrange;
+    if(currentGame != nullptr) {
+        return currentGame->objectData.data[itemID][originalHouseID].viewrange;
+    }
+    return 2; // Default fallback view range
 }
 
 int ObjectBase::getAreaGuardRange() const {
@@ -532,15 +550,24 @@ int ObjectBase::getAreaGuardRange() const {
 }
 
 int ObjectBase::getWeaponRange() const {
-    return currentGame->objectData.data[itemID][originalHouseID].weaponrange;
+    if(currentGame != nullptr) {
+        return currentGame->objectData.data[itemID][originalHouseID].weaponrange;
+    }
+    return 0; // Default fallback weapon range
 }
 
 int ObjectBase::getWeaponReloadTime() const {
-    return currentGame->objectData.data[itemID][originalHouseID].weaponreloadtime;
+    if(currentGame != nullptr) {
+        return currentGame->objectData.data[itemID][originalHouseID].weaponreloadtime;
+    }
+    return 100; // Default fallback reload time
 }
 
 int ObjectBase::getInfSpawnProp() const {
-    return currentGame->objectData.data[itemID][originalHouseID].infspawnprop;
+    if(currentGame != nullptr) {
+        return currentGame->objectData.data[itemID][originalHouseID].infspawnprop;
+    }
+    return 0; // Default fallback infantry spawn probability
 }
 
 ObjectBase* ObjectBase::createObject(int itemID, House* Owner, bool byScenario) {
@@ -673,6 +700,10 @@ ObjectBase* ObjectBase::loadObject(InputStream& stream, int itemID, Uint32 objec
 }
 
 bool ObjectBase::targetInWeaponRange() const {
+    if(currentGame == nullptr || target.getObjPointer() == nullptr) {
+        return false;
+    }
+    
     Coord coord = (target.getObjPointer())->getClosestPoint(location);
     FixPoint dist = blockDistance(location,coord);
 
