@@ -36,6 +36,7 @@
 #include <GUI/dune/DuneStyle.h>
 
 #include <Menu/MainMenu.h>
+#include <Menu/OptionsMenu.h>
 
 #include <misc/fnkdat.h>
 #include <misc/FileSystem.h>
@@ -103,7 +104,6 @@ inline std::string demangleSymbol(const char* symbolname) {
 
 void setVideoMode(int displayIndex);
 void realign_buttons();
-void createWhiteCursor();
 
 static void printUsage() {
     fprintf(stderr, "Usage:\n\tdunelegacy [--showlog] [--fullscreen|--window] [--PlayerName=X] [--ServerPort=X]\n");
@@ -134,17 +134,42 @@ void setVideoMode(int displayIndex)
         SDL_Log("Warning: Falling back to a display resolution of 640x480!");
         settings.video.physicalWidth = 640;
         settings.video.physicalHeight = 480;
-        settings.video.width = 640;
-        settings.video.height = 480;
+        int factor = getLogicalToPhysicalResolutionFactor(settings.video.physicalWidth, settings.video.physicalHeight);
+        // Prevent division by zero and ensure minimum dimensions
+        if(factor <= 0) {
+            factor = 1;
+        }
+        settings.video.width = settings.video.physicalWidth / factor;
+        settings.video.height = settings.video.physicalHeight / factor;
+        // Ensure minimum dimensions
+        if(settings.video.width < 640) settings.video.width = 640;
+        if(settings.video.height < 480) settings.video.height = 480;
     } else {
         settings.video.physicalWidth = closestDisplayMode.w;
         settings.video.physicalHeight = closestDisplayMode.h;
         int factor = getLogicalToPhysicalResolutionFactor(settings.video.physicalWidth, settings.video.physicalHeight);
+        // Prevent division by zero and ensure minimum dimensions
+        if(factor <= 0) {
+            factor = 1;
+        }
         settings.video.width = settings.video.physicalWidth / factor;
         settings.video.height = settings.video.physicalHeight / factor;
-
+        
+        // Ensure minimum dimensions
+        if(settings.video.width < 640) settings.video.width = 640;
+        if(settings.video.height < 480) settings.video.height = 480;
     }
 
+    // Prefer Direct3D on Windows, let SDL choose best renderer on other platforms
+#ifdef _WIN32
+    SDL_SetHint(SDL_HINT_RENDER_DRIVER, "direct3d11");
+#else
+    // On non-Windows platforms, let SDL choose the best renderer
+    SDL_SetHint(SDL_HINT_RENDER_DRIVER, "");
+#endif
+    SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "0");  // Use nearest-neighbor scaling for pixel-perfect look
+    SDL_SetHint(SDL_HINT_RENDER_BATCHING, "1");       // Enable render batching for performance
+
     window = SDL_CreateWindow("Dune Legacy",
                               SDL_WINDOWPOS_CENTERED_DISPLAY(displayIndex), SDL_WINDOWPOS_CENTERED_DISPLAY(displayIndex),
                               settings.video.physicalWidth, settings.video.physicalHeight,
@@ -160,38 +185,28 @@ void setVideoMode(int displayIndex)
     }
     SDL_RenderSetLogicalSize(renderer, settings.video.width, settings.video.height);
     screenTexture = SDL_CreateTexture(renderer, SCREEN_FORMAT, SDL_TEXTUREACCESS_TARGET, settings.video.width, settings.video.height);
-}
 
-void createWhiteCursor() {
-    // Create white arrow cursor
-    Uint8 data[4*32] = {0};  // All 0s for white
-    Uint8 mask[4*32] = {
-        0x80, 0x00, 0x00, 0x00,   // X.......
-        0xc0, 0x00, 0x00, 0x00,   // XX......
-        0xe0, 0x00, 0x00, 0x00,   // XXX.....
-        0xf0, 0x00, 0x00, 0x00,   // XXXX....
-        0xf8, 0x00, 0x00, 0x00,   // XXXXX...
-        0xfc, 0x00, 0x00, 0x00,   // XXXXXX..
-        0xfe, 0x00, 0x00, 0x00,   // XXXXXXX.
-        0xff, 0x00, 0x00, 0x00,   // XXXXXXXX
-        0xf8, 0x00, 0x00, 0x00,   // XXXXX...
-        0xb8, 0x00, 0x00, 0x00,   // X.XXX...
-        0x98, 0x00, 0x00, 0x00,   // X..XX...
-        0x0c, 0x00, 0x00, 0x00,   // ..XX....
-        0x0c, 0x00, 0x00, 0x00,   // ..XX....
-        0x06, 0x00, 0x00, 0x00,   // ..XX....
-        0x06, 0x00, 0x00, 0x00,   // .XX.....
-        0x03, 0x00, 0x00, 0x00,   // .X......
-    };
-
-    SDL_Cursor* cursor = SDL_CreateCursor(data, mask, 32, 16, 0, 0);
-    if(cursor) {
-        SDL_SetCursor(cursor);
+    // Check if texture creation failed
+    if (!screenTexture) {
+        fprintf(stderr, "SDL_CreateTexture failed: %s\n", SDL_GetError());
+        exit(EXIT_FAILURE);
     }
+
+    // Enable hardware acceleration
+    SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
+    SDL_SetTextureScaleMode(screenTexture, SDL_ScaleModeNearest);
+
+    SDL_ShowCursor(SDL_DISABLE);
 }
 
 void toogleFullscreen()
 {
+    // Safety checks
+    if(window == nullptr || renderer == nullptr) {
+        SDL_Log("Warning: Cannot toggle fullscreen - window or renderer not available");
+        return;
+    }
+
     if(SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN_DESKTOP) {
         // switch to windowed mode
         SDL_Log("Switching to windowed mode.");
@@ -203,21 +218,21 @@ void toogleFullscreen()
         // switch to fullscreen mode
         SDL_Log("Switching to fullscreen mode.");
         SDL_DisplayMode displayMode;
-        SDL_GetDesktopDisplayMode(SDL_GetWindowDisplayIndex(window), &displayMode);
-
-        SDL_SetWindowFullscreen(window, (SDL_GetWindowFlags(window) ^ SDL_WINDOW_FULLSCREEN_DESKTOP));
-
-        SDL_SetWindowSize(window, displayMode.w, displayMode.h);
-        SDL_RenderSetLogicalSize(renderer, settings.video.width, settings.video.height);
+        int displayIndex = SDL_GetWindowDisplayIndex(window);
+        if(displayIndex >= 0 && SDL_GetDesktopDisplayMode(displayIndex, &displayMode) == 0) {
+            SDL_SetWindowFullscreen(window, (SDL_GetWindowFlags(window) ^ SDL_WINDOW_FULLSCREEN_DESKTOP));
+            SDL_SetWindowSize(window, displayMode.w, displayMode.h);
+            SDL_RenderSetLogicalSize(renderer, settings.video.width, settings.video.height);
+        } else {
+            SDL_Log("Warning: Could not get desktop display mode, using current window size");
+            SDL_SetWindowFullscreen(window, (SDL_GetWindowFlags(window) ^ SDL_WINDOW_FULLSCREEN_DESKTOP));
+        }
     }
 
     // we just need to flush all events; otherwise we might get them twice
     SDL_PumpEvents();
     SDL_FlushEvents(SDL_FIRSTEVENT, SDL_LASTEVENT);
 
-    // Restore our custom cursor after event flush
-    createWhiteCursor();
-
     // wait a bit to avoid immediately switching back
     SDL_Delay(100);
 }
@@ -265,7 +280,9 @@ void createDefaultConfigFile(const std::string& configfilepath, const std::strin
                                 "Physical Width = 640\n"
                                 "Physical Height = 480\n"
                                 "Fullscreen = true\n"
-                                "FrameLimit = true           # Limit the frame rate to save energy?\n"
+                                "FrameLimit = false          # Limit the frame rate to save energy?\n"
+                                "VSync = false              # Enable vertical synchronization (limits FPS to monitor refresh rate)\n"
+
                                 "Preferred Zoom Level = 1    # 0 = no zooming, 1 = 2x, 2 = 3x\n"
                                 "Scaler = ScaleHD            # Scaler to use: ScaleHD = apply manual drawn mask to upscale, Scale2x = smooth edges, ScaleNN = nearest neighbour, \n"
                                 "RotateUnitGraphics = false  # Freely rotate unit graphics, e.g. carryall graphics\n"
@@ -513,7 +530,6 @@ int main(int argc, char *argv[]) {
         bool bFirstGamestart = false;
 
         debug = false;
-        cursorFrame = UI_CursorNormal;
 
         int currentDisplayIndex = SCREEN_DEFAULT_DISPLAYINDEX;
 
@@ -546,7 +562,8 @@ int main(int argc, char *argv[]) {
             settings.video.physicalWidth= myINIFile.getIntValue("Video","Physical Width",640);
             settings.video.physicalHeight = myINIFile.getIntValue("Video","Physical Height",480);
             settings.video.fullscreen = myINIFile.getBoolValue("Video","Fullscreen",false);
-            settings.video.frameLimit = myINIFile.getBoolValue("Video","FrameLimit",true);
+            settings.video.frameLimit = myINIFile.getBoolValue("Video","FrameLimit",false);
+            settings.video.vSync = myINIFile.getBoolValue("Video","VSync",false);
             settings.video.preferredZoomLevel = myINIFile.getIntValue("Video","Preferred Zoom Level", 0);
             settings.video.scaler = myINIFile.getStringValue("Video","Scaler","ScaleHD");
             settings.video.rotateUnitGraphics = myINIFile.getBoolValue("Video","RotateUnitGraphics",false);
@@ -615,36 +632,21 @@ int main(int argc, char *argv[]) {
             if(bFirstInit == true) {
                 SDL_Log("Initializing game...");
 
-                // Platform-specific renderer selection
-                #if defined(_WIN32)
-                    SDL_SetHint(SDL_HINT_RENDER_DRIVER, "direct3d");
-                #else
-                    SDL_SetHint(SDL_HINT_RENDER_DRIVER, "opengl");
-                #endif
-
+                // Force OpenGL rendering on macOS
                 SDL_SetHint(SDL_HINT_FRAMEBUFFER_ACCELERATION, "1");
-                SDL_SetHint(SDL_HINT_RENDER_VSYNC, "1");
-                SDL_SetHint(SDL_HINT_RENDER_BATCHING, "1");
-                SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "0"); // Use "1" for smooth scaling if desired
-                SDL_SetHint(SDL_HINT_RENDER_LINE_METHOD, "0");
+                SDL_SetHint(SDL_HINT_VIDEO_MAC_FULLSCREEN_SPACES, "0");
                 SDL_SetHint(SDL_HINT_VIDEO_ALLOW_SCREENSAVER, "0");
-
-                #if defined(__APPLE__)
-                    SDL_SetHint(SDL_HINT_VIDEO_MAC_FULLSCREEN_SPACES, "0");
-                    SDL_SetHint(SDL_HINT_VIDEO_HIGHDPI_DISABLED, "1");
-                #endif
-
-                // Optional: Disable EGL on X11 (Linux)
-                #if defined(__linux__)
-                    SDL_SetHint(SDL_HINT_VIDEO_X11_FORCE_EGL, "0");
-                #endif
+                SDL_SetHint(SDL_HINT_VIDEO_HIGHDPI_DISABLED, "1");
+                SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "1");
+                SDL_SetHint(SDL_HINT_RENDER_VSYNC, settings.video.vSync ? "1" : "0");
+                SDL_SetHint(SDL_HINT_VIDEO_X11_FORCE_EGL, "0");  // Disable EGL
+                SDL_SetHint(SDL_HINT_RENDER_BATCHING, "1");      // Enable render batching
+                SDL_SetHint(SDL_HINT_RENDER_LINE_METHOD, "3");   // Best line rendering quality
 
                 if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO) < 0) {
                     THROW(sdl_error, "Couldn't initialize SDL: %s!", SDL_GetError());
                 }
 
-                createWhiteCursor();
-
                 SDL_version compiledVersion;
                 SDL_version linkedVersion;
                 SDL_VERSION(&compiledVersion);
@@ -704,10 +706,19 @@ int main(int argc, char *argv[]) {
 
             SDL_Log("Setting video mode...");
             setVideoMode(currentDisplayIndex);
+            
+            // Give the renderer time to fully initialize
+            SDL_Delay(100);
+            
             SDL_RendererInfo rendererInfo;
             SDL_GetRendererInfo(renderer, &rendererInfo);
             SDL_Log("Renderer: %s (max texture size: %dx%d)", rendererInfo.name, rendererInfo.max_texture_width, rendererInfo.max_texture_height);
 
+            // Verify renderer is valid before proceeding
+            if(renderer == nullptr) {
+                SDL_Log("Error: Renderer is null after setVideoMode!");
+                THROW(std::runtime_error, "Failed to create renderer during video mode initialization");
+            }
 
             SDL_Log("Loading fonts...");
             pFontManager = std::make_unique<FontManager>();
@@ -756,10 +767,18 @@ int main(int argc, char *argv[]) {
 
             bFirstInit = false;
 
+            // Re-enable cursor for main menu (fixes Windows cursor visibility issue)
+            SDL_ShowCursor(SDL_ENABLE);
+
             SDL_Log("Starting main menu...");
             { // Scope
-                if (MainMenu().showMenu() == MENU_QUIT_DEFAULT) {
+                int menuResult = MainMenu().showMenu();
+                if (menuResult == MENU_QUIT_DEFAULT) {
                     bExitGame = true;
+                } else if (menuResult == MENU_QUIT_REINITIALIZE) {
+                    // Reinitialize video mode and continue the loop
+                    SDL_Log("Reinitializing video mode...");
+                    // The loop will continue and reinitialize everything
                 }
             }
 
@@ -775,7 +794,13 @@ int main(int argc, char *argv[]) {
                 Mix_CloseAudio();
             } else {
                 // save the current display index for later reuse
-                currentDisplayIndex = SDL_GetWindowDisplayIndex(window);
+                if(window != nullptr) {
+                    currentDisplayIndex = SDL_GetWindowDisplayIndex(window);
+                    // Safety check: ensure we got a valid display index
+                    if(currentDisplayIndex < 0) {
+                        currentDisplayIndex = 0; // Fallback to primary display
+                    }
+                }
             }
 
             pTextManager.reset();
@@ -784,9 +809,19 @@ int main(int argc, char *argv[]) {
             pFontManager.reset();
             pFileManager.reset();
 
-            SDL_DestroyTexture(screenTexture);
-            SDL_DestroyRenderer(renderer);
-            SDL_DestroyWindow(window);
+            // Safely destroy SDL objects
+            if(screenTexture != nullptr) {
+                SDL_DestroyTexture(screenTexture);
+                screenTexture = nullptr;
+            }
+            if(renderer != nullptr) {
+                SDL_DestroyRenderer(renderer);
+                renderer = nullptr;
+            }
+            if(window != nullptr) {
+                SDL_DestroyWindow(window);
+                window = nullptr;
+            }
 
             if(bExitGame == true) {
                 TTF_Quit();
