@@ -375,11 +375,16 @@ void UnitBase::deviate(House* newOwner) {
     // Adding this in as a surrogate for damage inflicted upon deviation.. Still not sure what the best value
     // should be... going in with a 25% of the units value unless its a devastator which we can destruct or an ornithoper
     // which is likely to get killed
-    if(getItemID() == Unit_Devastator || getItemID() == Unit_Ornithopter){
-        newOwner->informHasDamaged(Unit_Deviator, currentGame->objectData.data[getItemID()][newOwner->getHouseID()].price);
-    } else{
-        newOwner->informHasDamaged(Unit_Deviator, currentGame->objectData.data[getItemID()][newOwner->getHouseID()].price / 10);
+    if(currentGame != nullptr) {
+        if(getItemID() == Unit_Devastator || getItemID() == Unit_Ornithopter){
+            newOwner->informHasDamaged(Unit_Deviator, currentGame->objectData.data[getItemID()][newOwner->getHouseID()].price);
+        } else{
+            newOwner->informHasDamaged(Unit_Deviator, currentGame->objectData.data[getItemID()][newOwner->getHouseID()].price / 10);
+        }
     }
+
+
+
 }
 
 void UnitBase::drawSelectionBox() {
@@ -942,27 +947,35 @@ void UnitBase::handleDamage(int damage, Uint32 damagerID, House* damagerOwner) {
 
     ObjectBase::handleDamage(damage, damagerID, damagerOwner);
 
+    if(currentGame == nullptr) {
+        return;
+    }
+    
     ObjectBase* pDamager = currentGame->getObjectManager().getObject(damagerID);
 
     if(pDamager != nullptr){
-        // Handle HUNT mode behavior
+
         if(attackMode == HUNT && !forced) {
-            ObjectBase* pDamager = currentGame->getObjectManager().getObject(damagerID);
             if(canAttack(pDamager)) {
                 if(!target || target.getObjPointer() == nullptr || !isInWeaponRange(target.getObjPointer())) {
                     // no target or target not on weapon range => switch target
                     doAttackObject(pDamager, false);
                 }
+
             }
         }
 
-        // Track damage specifically for deviators since they have special behavior
-        if(pDamager->getItemID() == Unit_Deviator) {
-            if(damagerOwner == getOwner()){
-                damage *= -1;
-            }
-            damagerOwner->informHasDamaged(pDamager->getItemID(), damage);
+        /*
+         This method records the damage taken so that QuantBot can use it to know how effective different unit
+         classes are during the current game so that it can adjust its unit build ratios
+        */
+
+        // If you damaged your own unit then, the damage should be treated as negative.
+        if(damagerOwner == getOwner()){
+            damage *= -1;
         }
+
+        damagerOwner->informHasDamaged(pDamager->getItemID(), damage);
     }
 }
 
@@ -1050,7 +1063,7 @@ bool UnitBase::isInWeaponRange(const ObjectBase* object) const {
         return false;
     }
 
-    Coord targetLocation = target.getObjPointer()->getClosestPoint(location);
+    Coord targetLocation = object->getClosestPoint(location);
 
     return (blockDistance(location, targetLocation) <= getWeaponRange());
 }
